#include		"Matrix.h"

clMatrix::clMatrix( void )
{
	memset( &m, 0, sizeof( m ) );

	m[ 0  ] = 1.0f;
	m[ 5  ] = 1.0f;
	m[ 10 ] = 1.0f;
	m[ 15 ] = 1.0f;
}

void clMatrix::operator = ( clMatrix m )
{
	memcpy( &this->m, &m, sizeof( m ) );
}

clMatrix clMatrix::operator * ( clMatrix n )
{
	clMatrix	r;

	r.m[ 0  ] = m[ 0  ] * n.m[ 0 ] + m[ 1  ] * n.m[ 4 ] + m[ 2  ] * n.m[ 8  ] + m[ 3  ] * n.m[ 12 ];
	r.m[ 1  ] = m[ 0  ] * n.m[ 1 ] + m[ 1  ] * n.m[ 5 ] + m[ 2  ] * n.m[ 9  ] + m[ 3  ] * n.m[ 13 ];
	r.m[ 2  ] = m[ 0  ] * n.m[ 2 ] + m[ 1  ] * n.m[ 6 ] + m[ 2  ] * n.m[ 10 ] + m[ 3  ] * n.m[ 14 ];
	r.m[ 3  ] = m[ 0  ] * n.m[ 3 ] + m[ 1  ] * n.m[ 7 ] + m[ 2  ] * n.m[ 11 ] + m[ 3  ] * n.m[ 15 ];
	r.m[ 4  ] = m[ 4  ] * n.m[ 0 ] + m[ 5  ] * n.m[ 4 ] + m[ 6  ] * n.m[ 8  ] + m[ 7  ] * n.m[ 12 ];
	r.m[ 5  ] = m[ 4  ] * n.m[ 1 ] + m[ 5  ] * n.m[ 5 ] + m[ 6  ] * n.m[ 9  ] + m[ 7  ] * n.m[ 13 ];
	r.m[ 6  ] = m[ 4  ] * n.m[ 2 ] + m[ 5  ] * n.m[ 6 ] + m[ 6  ] * n.m[ 10 ] + m[ 7  ] * n.m[ 14 ];
	r.m[ 7  ] = m[ 4  ] * n.m[ 3 ] + m[ 5  ] * n.m[ 7 ] + m[ 6  ] * n.m[ 11 ] + m[ 7  ] * n.m[ 15 ];
	r.m[ 8  ] = m[ 8  ] * n.m[ 0 ] + m[ 9  ] * n.m[ 4 ] + m[ 10 ] * n.m[ 8  ] + m[ 11 ] * n.m[ 12 ];
	r.m[ 9  ] = m[ 8  ] * n.m[ 1 ] + m[ 9  ] * n.m[ 5 ] + m[ 10 ] * n.m[ 9  ] + m[ 11 ] * n.m[ 13 ];
	r.m[ 10 ] = m[ 8  ] * n.m[ 2 ] + m[ 9  ] * n.m[ 6 ] + m[ 10 ] * n.m[ 10 ] + m[ 11 ] * n.m[ 14 ];
	r.m[ 11 ] = m[ 8  ] * n.m[ 3 ] + m[ 9  ] * n.m[ 7 ] + m[ 10 ] * n.m[ 11 ] + m[ 11 ] * n.m[ 15 ];
	r.m[ 12 ] = m[ 12 ] * n.m[ 0 ] + m[ 13 ] * n.m[ 4 ] + m[ 14 ] * n.m[ 8  ] + m[ 15 ] * n.m[ 12 ];
	r.m[ 13 ] = m[ 12 ] * n.m[ 1 ] + m[ 13 ] * n.m[ 5 ] + m[ 14 ] * n.m[ 9  ] + m[ 15 ] * n.m[ 13 ];
	r.m[ 14 ] = m[ 12 ] * n.m[ 2 ] + m[ 13 ] * n.m[ 6 ] + m[ 14 ] * n.m[ 10 ] + m[ 15 ] * n.m[ 14 ];
	r.m[ 15 ] = m[ 12 ] * n.m[ 3 ] + m[ 13 ] * n.m[ 7 ] + m[ 14 ] * n.m[ 11 ] + m[ 15 ] * n.m[ 15 ];
  
	return r;
}

void clMatrix::clClear( void )
{
	memset( &m, 0, sizeof( m ) );

	m[ 0  ] = 1.0f;
	m[ 5  ] = 1.0f;
	m[ 10 ] = 1.0f;
	m[ 15 ] = 1.0f;
}

void clMatrix::clCreateXRot( float fa )
{
	float	fcx = cos( fa );
	float	fsx = sin( fa );

	memset( &m, 0, sizeof( m ) );

	m[ 5  ] =   fcx;
	m[ 6  ] =   fsx;
	m[ 9  ] = - fsx;
	m[ 10 ] =   fcx;
	m[ 0  ] =   1.0f;
	m[ 15 ] =   1.0f;
}

void clMatrix::clCreateYRot( float fa )
{
	float	fcy = cos( fa );
	float	fsy = sin( fa );

	memset( &m, 0, sizeof( m ) );

	m[ 0  ] =   fcy;
	m[ 8  ] =   fsy;
	m[ 2  ] = - fsy;
	m[ 10 ] =   fcy;
	m[ 5  ] =   1.0f;
	m[ 15 ] =   1.0f;
}

void clMatrix::clCreateZRot( float fa )
{
	float	fcz = cos( fa );
	float	fsz = sin( fa );

	memset( &m, 0, sizeof( m ) );

	m[ 0  ] =   fcz;
	m[ 1  ] =   fsz;
	m[ 4  ] = - fsz;
	m[ 5  ] =   fcz;
	m[ 10 ] =   1.0f;
	m[ 15 ] =   1.0f;
}

void clMatrix::clCreateXYZRot( float fx, float fy, float fz )
{
	float	cx = cos( fx );
	float	sx = sin( fx );
	float	cy = cos( fy );
	float	sy = sin( fy );
	float	cz = cos( fz );
	float	sz = sin( fz );

	m[ 0  ] =   cz * cy;
	m[ 1  ] =   sz * cx + cz * sy * sx;
	m[ 2  ] =   sz * sx - cz * sy * cx;
	m[ 3  ] =   0.0f;
	m[ 4  ] = - sz * cy;
	m[ 5  ] =   cz * cx - sz * sy * sx;
	m[ 6  ] =   cz * sx + sz * sy * cx;
	m[ 7  ] =   0.0f;
	m[ 8  ] =   sy;
	m[ 9  ] = - cy * sx;
	m[ 10 ] =   cy * cx;
	m[ 11 ] =   0.0f;
	m[ 12 ] =   0.0f;
	m[ 13 ] =   0.0f;
	m[ 14 ] =   0.0f;
	m[ 15 ] =   1.0f;
}