#include	<math.h>

#include	"Vector.h"

/*
 * =================================================================================================
 * Vector 2d
 * =================================================================================================
 */

c_vector2d::c_vector2d( void )
{
	x = y = 0.0f;
}

c_vector2d::c_vector2d( float fx, float fy )
{
	x = fx;
	y = fy;
}

void c_vector2d::operator += ( c_vector2d vVector )
{
	x += vVector.x;
	y += vVector.y;
}

void c_vector2d::operator -= ( c_vector2d vVector )
{
	x -= vVector.x;
	y -= vVector.y;
}

void c_vector2d::operator *= ( c_vector2d vVector )
{
	x *= vVector.x;
	y *= vVector.y;
}

void c_vector2d::operator /= ( c_vector2d vVector )
{
	x /= vVector.x;
	y /= vVector.y;
}

void c_vector2d::operator += ( float fValue )
{
	x += fValue;
	y += fValue;
}

void c_vector2d::operator -= ( float fValue )
{
	x -= fValue;
	y -= fValue;
}

void c_vector2d::operator *= ( float fValue )
{
	x *= fValue;
	y *= fValue;
}

void c_vector2d::operator /= ( float fValue )
{
	x /= fValue;
	y /= fValue;
}

void c_vector2d::operator =	( c_vector3d vVector )
{
	x = vVector.x;
	y = vVector.y;
}

float c_vector2d::operator [] ( int iIndex )
{
	return ( &x )[ iIndex ];
}

c_vector2d c_vector2d::operator + ( c_vector2d vVector )
{
	return c_vector2d( x + vVector.x, y + vVector.y );
}

c_vector2d c_vector2d::operator - ( c_vector2d vVector )
{
	return c_vector2d( x - vVector.x, y - vVector.y );
}

c_vector2d c_vector2d::operator * ( c_vector2d vVector )
{
	return c_vector2d( x * vVector.x, y * vVector.y );
}

c_vector2d c_vector2d::operator / ( c_vector2d vVector )
{
	return c_vector2d( x / vVector.x, y / vVector.y );
}

c_vector2d c_vector2d::operator + ( float fValue )
{
	return c_vector2d( x + fValue, y + fValue );
}

c_vector2d c_vector2d::operator - ( float fValue )
{
	return c_vector2d( x - fValue, y - fValue );
}

c_vector2d c_vector2d::operator * ( float fValue )
{
	return c_vector2d( x * fValue, y * fValue );
}

c_vector2d c_vector2d::operator / ( float fValue )
{
	return c_vector2d( x / fValue, y / fValue );
}

void c_vector2d::VecSet( float fx, float fy )
{
	x = fx;
	y = fy;
}

void c_vector2d::VecZero( void )
{
	x = y = 0.0f;
}

void c_vector2d::VecNormalize( void )
{
	float	fLength = sqrt( x * x + y * y );

	if( fLength )
	{
		x /= fLength;
		y /= fLength;
	}
}

float c_vector2d::DotProduct( c_vector2d vVector )
{
	return ( x * vVector.x + y * vVector.y );
}

float c_vector2d::CosBetween( c_vector2d vVector )
{
	float	fDelx = x - vVector.x;
	float	fDely = y - vVector.y;

	float	fLength = sqrt( fDelx * fDelx + fDely * fDely );

	if( fLength )
		return ( ( x * vVector.x + y * vVector.y ) / fLength );
	else
		return ( 0.0f );
}

c_vector2d c_vector2d::PerpVector( void )
{
	return c_vector2d( - y, x );
}

c_vector2d c_vector2d::GetNormal( void )
{
	float	fLength  = 1 / sqrt( x * x + y * y );

	return c_vector2d( - y * fLength, x * fLength );
}

float c_vector2d::GetLength( void )
{
	return sqrt( x * x + y * y );
}

float c_vector2d::GetQLength( void )
{
	return (x * x + y * y);
}

void c_vector2d::VecReflect( c_vector2d vVector )
{
	float		fDot = this->DotProduct( vVector );

	x -= vVector.x * 2 * fDot;
	y -= vVector.y * 2 * fDot;
}

void c_vector2d::VecRotate( float fAngle )
{
	float		t = x;
	x = x * cos( fAngle ) - y * sin( fAngle );
	y = t * sin( fAngle ) + y * cos( fAngle );
}

/*
 * =================================================================================================
 * Vector 3d
 * =================================================================================================
 */

c_vector3d::c_vector3d( void )
{
	x = y = z = 0.0f;
}

c_vector3d::c_vector3d( float fx, float fy, float fz )
{
	x = fx;
	y = fy;
	z = fz;
}

void c_vector3d::operator += ( c_vector3d vVector )
{
	x += vVector.x;
	y += vVector.y;
	z += vVector.z;
}

void c_vector3d::operator -= ( c_vector3d vVector )
{
	x -= vVector.x;
	y -= vVector.y;
	z -= vVector.z;
}

void c_vector3d::operator *= ( c_vector3d vVector )
{
	x *= vVector.x;
	y *= vVector.y;
	z *= vVector.z;
}

void c_vector3d::operator /= ( c_vector3d vVector )
{
	x /= vVector.x;
	y /= vVector.y;
	z /= vVector.z;
}

void c_vector3d::operator += ( float fValue )
{
	x += fValue;
	y += fValue;
	z += fValue;
}

void c_vector3d::operator -= ( float fValue )
{
	x -= fValue;
	y -= fValue;
	z -= fValue;
}

void c_vector3d::operator *= ( float fValue )
{
	x *= fValue;
	y *= fValue;
	z *= fValue;
}

void c_vector3d::operator /= ( float fValue )
{
	x /= fValue;
	y /= fValue;
	z /= fValue;
}

c_vector3d c_vector3d::operator + ( c_vector3d vVector )
{
	return c_vector3d( x + vVector.x, y + vVector.y, z + vVector.z );
}

c_vector3d c_vector3d::operator - ( c_vector3d vVector )
{
	return c_vector3d( x - vVector.x, y - vVector.y, z - vVector.z );
}

c_vector3d c_vector3d::operator * ( c_vector3d vVector )
{
	return c_vector3d( x * vVector.x, y * vVector.y, z * vVector.z );
}

c_vector3d c_vector3d::operator / ( c_vector3d vVector )
{
	return c_vector3d( x / vVector.x, y / vVector.y, z / vVector.z );
}

c_vector3d c_vector3d::operator + ( float fValue )
{
	return c_vector3d( x + fValue, y + fValue, z + fValue );
}

c_vector3d c_vector3d::operator - ( float fValue )
{
	return c_vector3d( x - fValue, y - fValue, z - fValue );
}

c_vector3d c_vector3d::operator * ( float fValue )
{
	return c_vector3d( x * fValue, y * fValue, z * fValue );
}

c_vector3d c_vector3d::operator / ( float fValue )
{
	return c_vector3d( x / fValue, y / fValue, z / fValue );
}

void c_vector3d::operator =	( c_vector2d vVector )
{
	x = vVector.x;
	y = vVector.y;
	z = 0.0f;
}

float c_vector3d::operator [] ( int iIndex )
{
	return ( &x )[ iIndex ];
}

void c_vector3d::VecSet( float fx, float fy, float fz )
{
	x = fx;
	y = fy;
	z = fz;
}

void c_vector3d::VecZero( void )
{
	x = y = z = 0.0f;
}

void c_vector3d::VecNormalize( void )
{
	float	fLength = sqrt( x * x + y * y + z * z );

	x /= fLength;
	y /= fLength;
	z /= fLength;
}

float c_vector3d::DotProduct( c_vector3d vVector )
{
	return ( x * vVector.x + y * vVector.y + z * vVector.z );
}

float c_vector3d::CosBetween( c_vector3d vVector )
{
	float	fDelx = x - vVector.x;
	float	fDely = y - vVector.y;
	float	fDelz = z - vVector.z;

	float	fLength = sqrt( fDelx * fDelx + fDely * fDely + fDelz * fDelz );

	if( fLength )
		return ( ( x * vVector.x + y * vVector.y + z * vVector.z ) / fLength );
	else
		return ( 0.0f );
}

c_vector3d c_vector3d::CrossProduct( c_vector3d vVector )
{
	float	fx = ( y * vVector.z - vVector.y * z);
	float	fy = ( z * vVector.x - vVector.z * x);
	float	fz = ( x * vVector.y - vVector.x * y);

	return c_vector3d( fx, fy, fz );
}

c_vector3d c_vector3d::GetNormal( c_vector3d vVector )
{
	float	fx = ( y * vVector.z - vVector.y * z);
	float	fy = ( z * vVector.x - vVector.z * x);
	float	fz = ( x * vVector.y - vVector.x * y);

	float	fLength  = 1 / sqrt( fx * fx + fy * fy + fz * fz );

	return c_vector3d( fx * fLength, fy * fLength, fz * fLength );
}

float c_vector3d::GetLength( void )
{
	return sqrt( x * x + y * y + z * z);
}

void c_vector3d::VecRotateX( float fAng )
{
	float		ty = y;
	float		tz = z;

	y = ty * cos( fAng ) - tz * sin( fAng );
	z = ty * sin( fAng ) + tz * cos( fAng );
}

void c_vector3d::VecRotateY( float fAng )
{
	float		tx = x;
	float		tz = z;

	x = tx * cos( fAng ) - tz * sin( fAng );
	z = tx * sin( fAng ) + tz * cos( fAng );
}

void c_vector3d::VecRotateZ( float fAng )
{
	float		tx = x;
	float		ty = y;

	x = tx * cos( fAng ) - ty * sin( fAng );
	y = tx * sin( fAng ) + ty * cos( fAng );
}

void c_vector3d::VecRotAxis( c_vector3d Axis, float fAngle )
{
	float	fSin = sin( fAngle );
	float	fCos = cos( fAngle );
	float	f1MC = 1 - fCos;

	float	m11 = ( f1MC * Axis.x * Axis.x ) + fCos;
	float	m12 = ( f1MC * Axis.x * Axis.y ) + ( Axis.z * fSin );
	float	m13 = ( f1MC * Axis.x * Axis.z ) - ( Axis.y * fSin );

	float	m21 = ( f1MC * Axis.y * Axis.x ) - ( Axis.z * fSin );
	float	m22 = ( f1MC * Axis.y * Axis.y ) + fCos;
	float	m23 = ( f1MC * Axis.y * Axis.z ) + ( Axis.x * fSin );

	float	m31 = ( f1MC * Axis.z * Axis.x ) + ( Axis.y * fSin );
	float	m32 = ( f1MC * Axis.z * Axis.y ) - ( Axis.x * fSin );
	float	m33 = ( f1MC * Axis.z * Axis.z ) + fCos;

	float	fTemp;
	c_vector3d		vTmp;

	vTmp.VecSet( x, y, z );

	fTemp = vTmp.x * m11 + vTmp.y * m12 + vTmp.z * m13;
	x = fTemp;

	fTemp = vTmp.x * m21 + vTmp.y * m22 + vTmp.z * m23;
	y = fTemp;

	fTemp = vTmp.x * m31 + vTmp.y * m32 + vTmp.z * m33;
	z = fTemp;
}

void c_vector3d::VecMatrix( clMatrix m )
{
	float	fxr = x * m.m[ 0 ] + y * m.m[ 1 ] + z * m.m[ 2  ] + m.m[ 3  ];
	float	fyr = x * m.m[ 4 ] + y * m.m[ 5 ] + z * m.m[ 6  ] + m.m[ 7  ];
	float	fzr = x * m.m[ 8 ] + y * m.m[ 9 ] + z * m.m[ 10 ] + m.m[ 11 ];

	x = fxr;
	y = fyr;
	z = fzr;
}

void c_vector3d::VecInvMatrix( clMatrix m )
{
	float	fxs = x - m.m[ 3  ];
	float	fys = y - m.m[ 7  ];
	float	fzs = z - m.m[ 11 ];
	
	x = fxs * m.m[ 0 ] + fys * m.m[ 4 ] + fzs * m.m[ 8  ];
	y = fxs * m.m[ 1 ] + fys * m.m[ 5 ] + fzs * m.m[ 9  ];
	z = fxs * m.m[ 2 ] + fys * m.m[ 6 ] + fzs * m.m[ 10 ];
}