#pragma		comment( lib, "opengl32.lib" )
#pragma		comment( lib, "glu32.lib" )
#pragma		comment( lib, "glaux.lib" )

#include	<windows.h>

#include	<gl/gl.h>
#include	<gl/glu.h>
#include	<gl/glaux.h>
#include	<gl/glext.h>

#include	<stdio.h>
#include	<conio.h>

#include	<math.h>

#include	<fstream.h>

#include	"Vector.h"
#include	"Texture.h"
#include	"Geometry.h"
#include	"Box.h"
#include	"Font.h"
#include	"Sprite.h"
#include	"Camera.h"
#include	"Item.h"
#include	"Player.h"
#include	"Collision.h"
#include	"Text.h"
#include	"Matrix.h"
#include	"MGFModel.h"

#define		GAME_CLASS	"Class of the game"
#define		GAME_TITLE	"Title of the game"

#define		GAME_INIT			0x001
#define		GAME_UPDATE			0x002

#define		MAX_BOXES_ON_ROOM	50
#define		MAX_ITEMS_ON_ROOM	10
#define		MAX_CAMERAS_ON_ROOM	10
#define		MAX_ROOM_COUNT		10

#define		BOX_WALL			0x001
#define		BOX_TOUCH_FIELD		0x002
#define		BOX_DOOR			0x003

class	mgcRoom
{
	
public:

	mgcCamera	acam[ MAX_CAMERAS_ON_ROOM ];
	int			iCameraCount;

	mgcBox		abox[ MAX_BOXES_ON_ROOM ];	// Geometry for collision
	int			iBoxCount;					// Count boxes

	mgcItem		aitm[ MAX_ITEMS_ON_ROOM ];	// Items
	int			iItemCount;					// Count items

	mgcPlayer	aplr;

	mgcGeometry	geo;

	bool		bRenderBoxesBone;

	unsigned int	uiActiveCamera;
	
				mgcRoom		( void );

	void		fLoadRoom	( char *chName );
	void		fRoomCall	( void );

	void		fDrawBoxes	( void );

private:

};

mgcRoom::mgcRoom( void )
{
}

void mgcRoom::fLoadRoom( char *chName )
{
	FILE	*infile;
	int		i, j, k;
	char		chTexName[ 256 ];

	infile = fopen( chName, "r" );

	if( infile )
	{
		fscanf( infile, "%i", &iCameraCount );

		for( i = 0; i < iCameraCount; i ++ )
		{
			// Texture for current room
			memset( &chTexName, 0, 256 );
			fscanf( infile, "%s", &chTexName );
			acam[ i ].spr.tex.tLoadBMP( chTexName );

			// Position cameras
			fscanf( infile, "%f %f %f", &acam[ i ].vAngle.x, &acam[ i ].vAngle.y, &acam[ i ].vAngle.z );
			fscanf( infile, "%f %f %f", &acam[ i ].vCoord.x, &acam[ i ].vCoord.y, &acam[ i ].vCoord.z );

			acam[ i ].spr.bBlend   = false;
			acam[ i ].spr.bTexture = true;
			acam[ i ].spr.col[ 0 ] = 1.0f;
			acam[ i ].spr.col[ 1 ] = 1.0f;
			acam[ i ].spr.col[ 2 ] = 1.0f;
			acam[ i ].spr.col[ 3 ] = 1.0f;
			acam[ i ].spr.vPos[ 0 ].x = - 1.0f;
			acam[ i ].spr.vPos[ 0 ].y = - 1.0f;
			acam[ i ].spr.vPos[ 0 ].z = - 1.0f;
			acam[ i ].spr.vPos[ 1 ].x = + 1.0f;
			acam[ i ].spr.vPos[ 1 ].y = + 1.0f;
			acam[ i ].spr.vPos[ 1 ].z = - 1.0f;
		}

		// !!!READ GEOMETRY FROM FILE!!!
		fscanf( infile, "%i", &iBoxCount );	// Count active elements on room
			
		for( j = 0; j < iBoxCount; j ++ )
		{
			// Read geometry box
			fscanf( infile, "%f %f %f %f %f %f %i",
				    &abox[ j ].vPos.x, &abox[ j ].vPos.y, &abox[ j ].vPos.z,
					&abox[ j ].vSiz.x, &abox[ j ].vSiz.y, &abox[ j ].vSiz.z,
					&abox[ j ].iType );

			// Read special parameters for boxes
			switch( abox[ j ].iType )
			{
			case BOX_TOUCH_FIELD:
				fscanf( infile, "%i", &abox[ j ].uiCameraNext );

				break;
			case BOX_DOOR:
				fscanf( infile, "%s", &abox[ j ].chText );

				for( k = 0; k < strlen( abox[ j ].chText ); k ++ )
				{
					if( abox[ j ].chText[ k ] == '_' )
					{
						abox[ j ].chText[ k ] = ' ';
					}
				}

				break;
			}
		}
		// !!!END READ GEOMETRY FROM FILE!!!

		// !!!READ ITEMS FROM FILE!!!
		fscanf( infile, "%i", &iItemCount );

		for( j = 0; j < iItemCount; j ++ )
		{
			fscanf( infile, "%f %f %f %f %f %f", &aitm[ j ].spr.vPos[ 0 ].x,
												 &aitm[ j ].spr.vPos[ 0 ].y,
												 &aitm[ j ].spr.vPos[ 0 ].z,
												 &aitm[ j ].spr.vPos[ 1 ].x,
												 &aitm[ j ].spr.vPos[ 1 ].y,
												 &aitm[ j ].spr.vPos[ 1 ].z );

			memset( &chTexName, 0, 256 );
			fscanf( infile, "%s", &chTexName );
			aitm[ j ].spr.tex.tLoadBMPA( chTexName );

			fscanf( infile, "%f %f %f %f", &aitm[ j ].spr.col[ 0 ],
										   &aitm[ j ].spr.col[ 1 ],
										   &aitm[ j ].spr.col[ 2 ],
										   &aitm[ j ].spr.col[ 3 ] );

			fscanf( infile, "%i %i", &aitm[ j ].spr.bBlend, &aitm[ j ].spr.bTexture );

			fscanf( infile, "%f %f %f %f %f %f", &aitm[ j ].box.vPos.x,
												 &aitm[ j ].box.vPos.y,
												 &aitm[ j ].box.vPos.z,
												 &aitm[ j ].box.vSiz.x,
												 &aitm[ j ].box.vSiz.y,
												 &aitm[ j ].box.vSiz.z );

			memset( &aitm[ j ].box.chText, 0, 256 );
			fscanf( infile, "%s", &aitm[ j ].box.chText );

			for( k = 0; k < strlen( aitm[ j ].box.chText ); k ++ )
				{
					if( aitm[ j ].box.chText[ k ] == '_' )
					{
						aitm[ j ].box.chText[ k ] = ' ';
					}
				}
		}
		// !!!END READ ITEMS FROM FILE!!!

		fclose( infile );
	}
}

void mgcRoom::fRoomCall( void )
{
	acam[ uiActiveCamera ].spr.fDraw();

	glRotatef( acam[ uiActiveCamera ].vAngle.x, 1.0f, 0.0f, 0.0f );
	glRotatef( acam[ uiActiveCamera ].vAngle.y, 0.0f, 1.0f, 0.0f );
	glRotatef( acam[ uiActiveCamera ].vAngle.z, 0.0f, 0.0f, 1.0f );
	glTranslatef( acam[ uiActiveCamera ].vCoord.x, acam[ uiActiveCamera ].vCoord.y, acam[ uiActiveCamera ].vCoord.z );

	for( int i = 0; i < iItemCount; i ++ )
	{
		aitm[ i ].fDraw();
	}

	if( bRenderBoxesBone )
	{
		fDrawBoxes();
	}
}

void mgcRoom::fDrawBoxes( void )
{
	int		i;

	for( i = 0; i < iBoxCount; i ++ )
	{
		geo.gDrawBox( abox[ i ] );
	}

	for( i = 0; i < iItemCount; i ++ )
	{
		aitm[ i ].fRenderBox();
	}

	aplr.fRenderBox();
}

class	mgcMain
{

public:

	int			iStatus;

	HGLRC		hRc;
	HDC			hDc;
	HWND		hWnd;

	mgcRoom		room[ MAX_ROOM_COUNT ];
	int			iRoomCount;

	unsigned int	uiRoomActive;

	float		fCurTime;
	float		fDelTime;
	float		fOldTime;
	float		fSpdGame;
	float		fResult;

	mgcText		text;

	mgcMGFModel	mgf;

	clMatrix	mat;

				mgcMain	( void );

	void		fLoadRooms	( char *chName );

	void		fUpdate		( void );
	void		fInitOpenGL	( void );
	void		fMoveUpdate	( void );

	void		fCollision		( unsigned int uiRoom );

	void		fGetItem	( unsigned int uiRoom, unsigned int uiItem );

private:

};

void mgcMain::fLoadRooms( char *chName )
{
	FILE	*infile;
	char	chFile[ 256 ];

	infile = fopen( "Room.txt", "r" );

	if( infile )
	{
		fscanf( infile, "%i", &iRoomCount );

		for( int i = 0; i < iRoomCount; i ++ )
		{
			memset( &chFile, 0, 256 );
			fscanf( infile, "%s", &chFile );

			room[ i ].fLoadRoom( chFile );
		}

		fclose( infile );
	}
}

void mgcMain::fMoveUpdate( void )
{
	mgf.bPlayAnimation = false;

	if( room[ uiRoomActive ].aplr.chKey[ VK_UP ] )
	{
		room[ uiRoomActive ].aplr.vPos += room[ uiRoomActive ].aplr.vDir * room[ uiRoomActive ].aplr.fSpeed * fResult;
		mgf.bPlayAnimation = true;
	}

	if( room[ uiRoomActive ].aplr.chKey[ VK_DOWN ] )
	{
		room[ uiRoomActive ].aplr.vPos -= room[ uiRoomActive ].aplr.vDir * room[ uiRoomActive ].aplr.fSpeed * fResult;
		mgf.bPlayAnimation = true;
	}

	if( room[ uiRoomActive ].aplr.chKey[ VK_LEFT ] )
	{
		room[ uiRoomActive ].aplr.fAngle -= 0.005f * fResult;
		mgf.bPlayAnimation = true;
	}

	if( room[ uiRoomActive ].aplr.chKey[ VK_RIGHT ] )
	{
		room[ uiRoomActive ].aplr.fAngle += 0.005f * fResult;
		mgf.bPlayAnimation = true;
	}

	if( !fSpdGame )
	{
		mgf.bPlayAnimation = false;
	}

	if( !mgf.bPlayAnimation && fSpdGame )
	{
		mgf.iCurrentFrame = 0;
	}
}

void mgcMain::fGetItem( unsigned int uiRoom, unsigned int uiItem )
{
	for( int i = 0; i < room[ uiRoom ].iItemCount; i ++ )
	{
		room[ uiRoom ].aitm[ i ] = room[ uiRoom ].aitm[ i + 1 ];
	}

	room[ uiRoom ].iItemCount --;
}

void mgcMain::fCollision( unsigned int uiRoom )
{
	mgcCollision	cls;
	c_vector3d		v3d;
	bool			bTextLeft = false;
	int				i;

	for( i = 0; i < room[ uiRoom ].iBoxCount; i ++ )
	{
		if( cls.fBoxBox( room[ uiRoomActive ].aplr.box, room[ uiRoom ].abox[ i ], v3d ) )
		{
			switch( room[ uiRoom ].abox[ i ].iType )
			{
			case BOX_WALL:
				room[ uiRoomActive ].aplr.vPos += v3d;

				break;
			case BOX_TOUCH_FIELD:
				room[ uiRoom ].uiActiveCamera = room[ uiRoom ].abox[ i ].uiCameraNext;

				break;
			case BOX_DOOR:
				if( room[ uiRoomActive ].aplr.chKey[ VK_SPACE ] && !bTextLeft )
				{
					text.fPut( room[ uiRoom ].abox[ i ].chText, 0 );

					bTextLeft = true;
				}

				break;
			}
		}
	}

	for( i = 0; i < room[ uiRoom ].iItemCount; i ++ )
	{
		if( cls.fBoxBox( room[ uiRoomActive ].aplr.box, room[ uiRoom ].aitm[ i ].box, v3d ) )
		{
			if( room[ uiRoomActive ].aplr.chKey[ VK_SPACE ] )
			{
				text.fPut( room[ uiRoom ].aitm[ i ].box.chText, 1 );

				fGetItem( uiRoom, i );
			}
		}
	}
}

mgcMain::mgcMain( void )
{
	iStatus = GAME_INIT;
	uiRoomActive    = 0;

	fSpdGame = 1.0f;
}

void mgcMain::fInitOpenGL( void )
{
	glViewport( 0, 0, 1024, 768 );
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glFrustum( -1.0f, 1.0f, -1.0f, 1.0f, 1.0f, 100.0f );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
}

void mgcMain::fUpdate( void )
{
	switch( iStatus )
	{
	case GAME_INIT:
		PIXELFORMATDESCRIPTOR	pfd;
		int						nPixelFormat;

		hDc = GetDC( hWnd );

		memset( &pfd, 0, sizeof( pfd ) );
		pfd.dwFlags  = PFD_DOUBLEBUFFER | PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL;
		nPixelFormat = ChoosePixelFormat( hDc, &pfd );
		SetPixelFormat( hDc, nPixelFormat, &pfd );

		hRc = wglCreateContext( hDc );

		wglMakeCurrent( hDc, hRc );

		fInitOpenGL();

		iStatus = GAME_UPDATE;

		fLoadRooms( "Rooms.txt" );

		// SHIT

		room[ uiRoomActive ].aplr.box.vPos.x = 0.0f;
		room[ uiRoomActive ].aplr.box.vPos.y = 0.0f;
		room[ uiRoomActive ].aplr.box.vPos.z = 0.0f;
		room[ uiRoomActive ].aplr.box.vSiz.x = 1.0f;
		room[ uiRoomActive ].aplr.box.vSiz.y = 5.0f;
		room[ uiRoomActive ].aplr.box.vSiz.z = 1.0f;
		room[ uiRoomActive ].aplr.fSpeed = 0.003f;

		text.fnt.fInit( hDc );
		text.fnt.fBuild( "Courier", 20, 30 );

		mgf.funLoadModel( "terror.txt" );
		mgf.funLoadAnimation( "run.txt" );

		mgf.funSetAnimation( 0, 22 );

		break;
	case GAME_UPDATE:

		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

		glLoadIdentity();

		room[ uiRoomActive ].fRoomCall();
		fMoveUpdate();
		room[ uiRoomActive ].aplr.fUpdate();
		fCollision( uiRoomActive );

		glTranslatef( room[ uiRoomActive ].aplr.vPos.x, room[ uiRoomActive ].aplr.vPos.y - 1.0f, room[ uiRoomActive ].aplr.vPos.z );
		glRotatef( - 90,1,0,0);
		glRotatef( - room[ uiRoomActive ].aplr.fAngle * 180 / 3.14 + 90, 0.0f, 0.0f, 1.0f );
		glScalef( 0.05f, 0.05f, 0.06f );

		//for( int i = 0; i < 100; i ++ )
		{
		mgf.funRender( fDelTime );
		}

		fSpdGame = !text.fUpdate( fDelTime, room[ uiRoomActive ].aplr.chKey[ VK_SPACE ] );

		SwapBuffers( hDc );

		break;
	}

	if( room[ uiRoomActive ].aplr.chKey[ VK_ESCAPE ] )
	{
		PostQuitMessage( 0 );
	}
}

mgcMain		mm;

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch( uMsg )
	{
	case WM_KEYUP:
		mm.room[ mm.uiRoomActive ].aplr.chKey[ wParam ] = false;

		break;
	case WM_KEYDOWN:
		mm.room[ mm.uiRoomActive ].aplr.chKey[ wParam ] = true;

		break;
	case WM_DESTROY:
		PostQuitMessage( 0 );

		break;
	case WM_QUIT:
		PostQuitMessage( 0 );

		break;
	default:
		return DefWindowProc( hWnd, uMsg, wParam, lParam );

		break;
	}
}

WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd )
{
	WNDCLASSEX		wClass;
	MSG				msg;

	memset( &wClass, 0, sizeof( wClass ) );
	memset( &msg, 0, sizeof( msg ) );
	
	wClass.cbSize = sizeof( wClass );
	wClass.lpszClassName = GAME_CLASS;
	wClass.lpfnWndProc = WndProc;

	RegisterClassEx( &wClass );

	mm.hWnd = CreateWindowEx( 0, GAME_CLASS, GAME_TITLE, WS_POPUP, 0, 0, 1024, 768, 0, 0, 0, 0 );

	ShowWindow( mm.hWnd, SW_SHOW );

	ShowCursor( false );

	mm.fOldTime = GetTickCount();

	while( msg.message != WM_QUIT )
	{
		if( PeekMessage( &msg, 0, 0, 0, PM_REMOVE ) )
		{
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
		else
		{
			mm.fCurTime = GetTickCount();
			mm.fDelTime = mm.fCurTime - mm.fOldTime;
			mm.fOldTime = mm.fCurTime;
			mm.fResult  = mm.fDelTime * mm.fSpdGame;
		
			mm.fUpdate();
		}
	}

	return 0;
}